﻿// Accord Statistics Library
// The Accord.NET Framework
// http://accord-framework.net
//
// Copyright © César Souza, 2009-2014
// cesarsouza at gmail.com
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

namespace Accord.Statistics.Distributions
{
    using Accord.Statistics.Distributions.Multivariate;
    using Accord.Statistics.Distributions.Univariate;
    using System.Runtime.CompilerServices;

    /// <summary>
    ///   Contains more than 40 statistical distributions, with support 
    ///   for most probability distribution measures and estimation methods.
    /// </summary>
    /// 
    /// <remarks>
    /// <para>
    ///   This namespace contains a huge collection of probability distributions, ranging the from the common
    ///   and simple <see cref="NormalDistribution">Normal (Gaussian) </see> and <see cref="PoissonDistribution">
    ///   Poisson</see> distributions to <see cref="InverseWishartDistribution">Inverse-Wishart</see> and 
    ///   <see cref="MultivariateMixture{T}">multivariate mixture distributions</see>, including many specialized
    ///   univariate distributions used in <see cref="Accord.Statistics.Testing">statistical hypothesis testing</see>.
    ///   Some of those distributions include the <see cref="MannWhitneyDistribution"/>, <see cref="TDistribution"/>,
    ///   <see cref="WilcoxonDistribution"/>, <see cref="KolmogorovSmirnovDistribution"/> and many others.</para>
    /// 
    /// <para>
    ///   For a complete list of all <see cref="Accord.Statistics.Distributions.Univariate">
    ///   univariate probability distributions</see>, check the <see cref="Accord.Statistics.Distributions.Univariate"/>
    ///   namespace. For a complete list of all <see cref="Accord.Statistics.Distributions.Multivariate">
    ///   multivariate distributions</see>, please see the <see cref="Accord.Statistics.Distributions.Multivariate"/>
    ///   namespace.</para>
    ///  
    /// <para>
    ///   A list of <see cref="Accord.Statistics.Distributions.DensityKernels.IDensityKernel">density kernels</see>
    ///   such as the <see cref="Accord.Statistics.Distributions.DensityKernels.GaussianKernel">Gaussian kernel</see>
    ///   and the <see cref="Accord.Statistics.Distributions.DensityKernels.EpanechnikovKernel">Epanechnikov kernel</see>
    ///   are available in the <see cref="Accord.Statistics.Distributions.DensityKernels"/> namespace. </para>
    /// 
    /// 
    /// <para>
    ///   The namespace class diagram is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Statistics.png" />
    ///   
    /// <para>
    ///   The namespace class diagram for univariate distributions is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Statistics.Univariate.png" />
    ///   
    /// <para>
    ///   The namespace class diagram for multivariate distributions is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Statistics.Multivariate.png" />
    /// </remarks>
    /// 
    ///   
    /// <seealso cref="Accord.Statistics.Distributions"/>
    /// <seealso cref="Accord.Statistics.Distributions.Univariate"/>
    /// <seealso cref="Accord.Statistics.Distributions.Multivariate"/>
    /// <seealso cref="Accord.Statistics.Distributions.Fitting"/>
    /// <seealso cref="Accord.Statistics.Distributions.DensityKernels"/>
    /// <seealso cref="Accord.Statistics.Testing"/>
    /// 
    [CompilerGenerated]
    class NamespaceDoc
    {
    }
}
